import unittest

class TestChannel(unittest.TestCase):
    def test_at_channel_create(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.at_channel_create())
        assert True # TODO: implement your test here

    def test_at_init(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.at_init())
        assert True # TODO: implement your test here

    def test_channel_prefix(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.channel_prefix(msg, emit))
        assert True # TODO: implement your test here

    def test_distribute_message(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.distribute_message(msg, online))
        assert True # TODO: implement your test here

    def test_format_external(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.format_external(msg, senders, emit))
        assert True # TODO: implement your test here

    def test_format_message(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.format_message(msg, emit))
        assert True # TODO: implement your test here

    def test_format_senders(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.format_senders(senders))
        assert True # TODO: implement your test here

    def test_message_transform(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.message_transform(msg, emit, prefix, sender_strings, external))
        assert True # TODO: implement your test here

    def test_msg(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.msg(msgobj, header, senders, sender_strings, persistent, online, emit, external))
        assert True # TODO: implement your test here

    def test_pose_transform(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.pose_transform(msg, sender_string))
        assert True # TODO: implement your test here

    def test_post_join_channel(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.post_join_channel(joiner))
        assert True # TODO: implement your test here

    def test_post_leave_channel(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.post_leave_channel(leaver))
        assert True # TODO: implement your test here

    def test_post_send_message(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.post_send_message(msg))
        assert True # TODO: implement your test here

    def test_pre_join_channel(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.pre_join_channel(joiner))
        assert True # TODO: implement your test here

    def test_pre_leave_channel(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.pre_leave_channel(leaver))
        assert True # TODO: implement your test here

    def test_pre_send_message(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.pre_send_message(msg))
        assert True # TODO: implement your test here

    def test_tempmsg(self):
        # channel = Channel()
        # self.assertEqual(expected, channel.tempmsg(message, header, senders))
        assert True # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()
